from cStringIO import StringIO
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.PloneWurfl.exceptions import *
import re

class PloneWurflTool(UniqueObject, SimpleItem):
    """ Plone Wurfl Tool.
    """

    id = 'portal_wurfl'
    meta_type = 'Plone Wurfl Tool'

    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)
    
    def __init__(self):
        UniqueObject.__init__(self)
        SimpleItem.__init__(self)

        # Regexes to remove noise from user agents
        self._noise = (("/SN\d{15}", "/SNXXXXXXXXXXXXXXX"),
                       ("UP.Link\/.*?(\s|$)", ""),
                       ("\(via IBM WBI \d+\.\d+\)", ""),
                       ("GMCC\/\d\.\d", ""))
        self._noise_re = [(re.compile(exp), repl) for exp, repl in self._noise]
        self._name_test_re = re.compile(r'^(_|[a-z])(_|[a-z]|[0-9])+$')

        self.we_installed_ploneazax = False
        
    def _getTree(self):
        pu = getToolByName(self, 'portal_url')
        tree = getattr(pu.getPortalObject(), 'generic', None)        
        # If not found use the first top-level tree
        if tree is None:
            obs = pu.getPortalObject().objectValues('MobileDevice')
            if obs:
                tree = obs[0]
        return tree

    def getDeviceByUserAgent(self, useragent, tree=None, returnActualDeviceRoot=False, 
        search=None, debug=False):
        """Return matching device"""
        
        pc = getToolByName(self, 'portal_catalog')        
        pu = getToolByName(self, 'portal_url')
        devua = useragent.strip()
        if debug:
            self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - useragent = %s" % devua)
        
        # If tree_root is None then use the generic one
        if tree is None:
            tree = self._getTree()
        if tree is None:
            raise TreeNotFound
        
        # Direct match
        brains = pc(portal_type='MobileDevice', 
            path="/".join(tree.getPhysicalPath()),
            getUserAgent=devua)
        if brains:
            if debug:
                self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - direct match")
            if returnActualDeviceRoot:
                return brains[0].getObject().getActualDeviceRoot()
            else:
                return brains[0].getObject()

        # Filter noise and try direct match again.   
        for exp, repl in self._noise_re:
            devua = exp.sub(repl, devua)
        devua = devua.strip()
        if debug:
            self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - filtered useragent = %s" % devua)

        # Direct match after noise reduction
        brains = pc(portal_type='MobileDevice', 
            path="/".join(tree.getPhysicalPath()),
            getUserAgent=devua)
        if brains:
            if debug:
                self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - direct match after filtering")
            if returnActualDeviceRoot:
                return brains[0].getObject().getActualDeviceRoot()
            else:
                return brains[0].getObject()
            
        # Search algorithm 
        if search is None:
            try:
                from Products.PloneWurfl.algorithms import JaroWinkler, LevenshteinDistance
            except:
                raise DeviceNotFound(useragent)
                
            # Default to JaroWinkler
            search = JaroWinkler()
            
            brains = pc(portal_type='MobileDevice', 
                path="/".join(tree.getPhysicalPath()))
            if debug:
                self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - try JaroWinkler")  
            device_brain = search(devua, brains)
            if device_brain is not None:
                if debug:
                    self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - JaroWinkler returns %s (actual device root = %s)" 
                        % (device_brain.getObject(), device_brain.getObject().getActualDeviceRoot()))
                if returnActualDeviceRoot:
                    return device_brain.getObject().getActualDeviceRoot()
                else:
                    return device_brain.getObject()

            # Argh. Now go for the closest match.
            search = LevenshteinDistance()
            if debug:
                self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - try LevenshteinDistance")
            device_brain = search(devua, brains)
            if device_brain is not None:
                if debug:
                    self.plone_log("[PloneWurflTool.getDeviceByUserAgent] - LevenshteinDistance returns %s (actual device root = %s)" 
                        % (device_brain.getObject(), device_brain.getObject().getActualDeviceRoot()))
                if returnActualDeviceRoot:
                    return device_brain.getObject().getActualDeviceRoot()
                else:
                    return device_brain.getObject()

        else:
            # User supplied search
            brains = pc(portal_type='MobileDevice', 
                path="/".join(tree.getPhysicalPath()))
            device_brain = search(devua, brains)
            if device_brain is not None:
                if returnActualDeviceRoot:
                    return device_brain.getObject().getActualDeviceRoot()
                else:
                    return device_brain.getObject()

        #raise DeviceNotFound(useragent)
        # Return generic device
        return tree
   
InitializeClass(PloneWurflTool)
