from Products.PloneWurfl.exceptions import DeviceNotFound

"""
Class definitions adapted from pywurfl by Armand Lynch <lyncha@users.sourceforge.net>
"""

class Algorithm(object):
    """
    Base class for all search algorithms
    """
    def __call__(self, ua, devices):
        """
        Every algorithm class must define a __call__ method.

        @param ua: The user agent
        @type ua: string
        @param devices: The devices brains to search
        @type devices: MobileDevice brain
        @rtype: MobileDevice brain
        """
        raise NotImplementedError


try:
    import Levenshtein

    class JaroWinkler(Algorithm):
        """
        Jaro-Winkler Search Algorithm
        """

        def __init__(self, accuracy=0.7, weight=0.05):
            """
            @param accuracy: The tolerance that the Jaro-Winkler algorithm will
                             use to determine if a user agent matches
                             0.0 >= accuracy <= 1.0
            @type accuracy: float
            """

            self.accuracy = accuracy
            self.weight = weight

        def __call__(self, ua, devices):
            """
            @param ua: The user agent
            @type ua: string
            @param devices: The devices brains to search
            @type devices: MobileDevice brain
            @rtype: MobileDevice brain
            """
            devuas_dict = {}
            for device_brain in devices:
                devuas_dict[device_brain.getUserAgent] = device_brain
                
            #import pdb;pdb.set_trace()
            match = max((Levenshtein.jaro_winkler(x, ua, self.weight), x) for
                        x in devuas_dict.keys())
            if match[0] >= self.accuracy:
                return devuas_dict[match[1]]
            else:
                return None

    class LevenshteinDistance(Algorithm):
        """
        Levenshtein distance Search Algorithm
        """

        def __call__(self, ua, devices):
            """
            @param ua: The user agent
            @type ua: string
            @param devices: The devices brains to search
            @type devices: MobileDevice brain
            @rtype: MobileDevice brain
            """
            devuas_dict = {}
            for device_brain in devices:
                devuas_dict[device_brain.getUserAgent] = device_brain

            match = max((Levenshtein.distance(ua, x), x) for x in
                        devuas_dict.keys())
            return devuas_dict[match[1]]

except ImportError:
    pass


